/*
  WiFi Web Server

 A simple web server that shows the value of the analog input pins.

 This example is written for a network using WPA encryption. For
 WEP or WPA, change the WiFi.begin() call accordingly.

 Circuit:
 * Analog inputs attached to pins A0 through A5 (optional)

 created 13 July 2010
 by dlf (Metodo2 srl)
 modified 31 May 2012
 by Tom Igoe


  Find the full UNO R4 WiFi Network documentation here:
  https://docs.arduino.cc/tutorials/uno-r4-wifi/wifi-examples#wi-fi-web-server
 */

#include "WiFiS3.h"
#include <malloc.h>

#include "base64.hpp"

//#include <SD.h>
#include <SdFat.h>                // SD card & FAT filesystem library
#include <SPI.h>

SdFat                SD;

// TFT display and SD card will share the hardware SPI interface.
// Hardware SPI pins are specific to the Arduino board type and
// cannot be remapped to alternate pins.  For Arduino Uno,
// Duemilanove, etc., pin 11 = MOSI, pin 12 = MISO, pin 13 = SCK.
#define SD_CS    4  // Chip select line for SD card on Shield
#define TFT_CS  10  // Chip select line for TFT display on Shield
#define TFT_DC   8  // Data/command line for TFT on Shield
#define TFT_RST  -1  // Reset line for TFT is handled by seesaw!

#define BMPFile "/test.bmp"
//#include "arduino_secrets.h" 
///////please enter your sensitive data in the Secret tab/arduino_secrets.h
//char ssid[] = SECRET_SSID;        // your network SSID (name)
//char pass[] = SECRET_PASS;    // your network password (use for WPA, or use as key for WEP)
char ssid[] = "??????????";//SECRET_SSID;        // your network SSID (name)
char pass[] = "?????";//SECRET_PASS;    // your network password (use for WPA, or use as key for WEP)

int keyIndex = 0;                 // your network key index number (needed only for WEP)

int status = WL_IDLE_STATUS;

WiFiServer server(80);

//unsigned char base64_text[60]={0};
//unsigned char decoded_tex[60];
#define N 8192
unsigned char *base64_text = (unsigned char *)malloc(N);
unsigned char *decoded_text = (unsigned char *)malloc(N);
int ik=0;

void SetupSD(){
    Serial.print(F("Initializing SD filesystem...\n"));
  if(!SD.begin(SD_CS, SD_SCK_MHZ(10))) { // Breakouts require 10 MHz limit due to longer wires
    Serial.println(F("SD begin() failed"));
    for(;;); // Fatal error, do not continue
  }
}
void SetupWIFI(){
  // check for the WiFi module:
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    // don't continue
    while (true);
  }

  String fv = WiFi.firmwareVersion();
  if (fv < WIFI_FIRMWARE_LATEST_VERSION) {
    Serial.println("Please upgrade the firmware");
  }

  // attempt to connect to WiFi network:
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);

    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);

    // wait 10 seconds for connection:
    delay(10000);
  }
  server.begin();
  // you're connected now, so print out the status:
  printWifiStatus();
  //Serial.println("Ready");
}

void setup() {
  //Initialize serial and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("SETUP Init");
    // start by disabling both SD and TFT
  pinMode(TFT_CS, OUTPUT);
  digitalWrite(TFT_CS, HIGH);
  pinMode(SD_CS, OUTPUT);
  digitalWrite(SD_CS, HIGH);

  SetupSD();
  SetupWIFI();
  Serial.println("SETUP Done");
}

void loop() {
  // listen for incoming clients
  WiFiClient client = server.available();
  if (client) {
    char c;
    Serial.print("new client connected: ");
    Serial.println(client.remoteIP());

    // an HTTP request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        c = client.read();
        base64_text[ik++]=c;
        Serial.write(c);
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the HTTP request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // send a standard HTTP response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");  // the connection will be closed after completion of the response
          client.println("Refresh: 5");  // refresh the page automatically every 5 sec
          client.println();
          client.println("<!DOCTYPE HTML>");
          client.println("<html>");
          // output the value of each analog input pin
          
          for (int analogChannel = 0; analogChannel < 6; analogChannel++) {
            int sensorReading = analogRead(analogChannel);
            client.print("analog input ");
            client.print(analogChannel);
            client.print(" is ");
            client.print(sensorReading);
            client.println("<br />");
          }
          client.println("</html>");
          break;
        }
        if (c == '\n') {
          // you're starting a new line
          currentLineIsBlank = true;
        } else if (c != '\r') {
          // you've gotten a character on the current line
          currentLineIsBlank = false;
        }
      }

    }
    // give the web browser time to receive the data
    delay(1);

    // close the connection:
    client.stop();
    Serial.println("client disconnected");
 
      if(ik>0){
        Serial.print("[");
        Serial.print(ik);
        Serial.println("]");
          for(int i=0; i < ik;i++)
          Serial.write(base64_text[i]);
         }
         int decoded_length = decode_base64(base64_text,decoded_text);           
         Serial.print("Decoded Text: ");Serial.println((char *)decoded_text);
         Serial.print("Decoded Length: ");Serial.println(decoded_length);
         mySaveFile(BMPFile,(char *)decoded_text,decoded_length);
         Serial.print("Saved img ");
         Serial.println(decoded_length);
         delay(500);
         //bmpDraw(BMPFile, 0, 0);
         //bmpTestView(BMPFile);//bmpTestView(BMPFile);
      free(base64_text);
      free(decoded_text);
      ik=0;
  }
}

void mySaveFile(char *filename,char *pTxt,int len){
  File fpw;

  if ((fpw = SD.open(filename, FILE_WRITE | O_TRUNC)) == NULL) {
    Serial.println("error opening test.txt");
    return;
  }
  for(int i=0; i < len; i++)
    fpw.write(pTxt[i]);
  
  fpw.close();
}

void printWifiStatus() {
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // print your board's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
}


